#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2004 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#    DrPython is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
import wx
import wx.stc
from drProperty import *
import drScrolledMessageDialog
from drPreferences import drPreferences

class drColorPanel(wx.Panel):

	def __init__(self, parent, id, point, size, colorstring, fg):
		wx.Panel.__init__(self, parent, id, point, size)

		self.ID_COLOR = 105
		self.ID_RED = 106
		self.ID_GREEN = 107
		self.ID_BLUE = 108

		self.txtColor = wx.TextCtrl(self, self.ID_COLOR, colorstring, wx.Point(0, 10), wx.Size(70, -1))

		self.isForeground = fg

		r = int(colorstring[1:3], 16)
		g = int(colorstring[3:5], 16)
		b = int(colorstring[5:7], 16)		

		self.red = wx.Slider(self, self.ID_RED, r, 0, 255, wx.Point(0, 40), wx.Size(100, -1), wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.green = wx.Slider(self, self.ID_GREEN, g, 0, 255, wx.Point(0, 70), wx.Size(100, -1), wx.SL_HORIZONTAL | wx.SL_LABELS )
		self.blue = wx.Slider(self, self.ID_BLUE, b, 0, 255, wx.Point(0, 100), wx.Size(100, -1), wx.SL_HORIZONTAL | wx.SL_LABELS )

		self.Bind(wx.EVT_SCROLL, self.OnScroll)
		self.Bind(wx.EVT_TEXT,  self.OnTextChange, id=self.ID_COLOR)

	def OnScroll(self, event):
		i = event.GetId()
		if (i == self.ID_RED):
			r = self.red.GetValue()
			sr = ''
			if (r < 16):
				sr = '0'
			sr = sr + hex(r)[2:]
			sr = sr.upper()
			text = self.txtColor.GetValue()
			if (len(text) is not 7):
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("The text box above should be formatted as follows:\n#0033AA,\nStart with the # character, followed by 6 characters which can be 0-9 or a-f.\nDrPython will ignore any changes to the color until this is fixed."), "Error")
				d.ShowModal()
				d.Destroy()	
				return
			self.txtColor.SetValue(text[0:1] + sr + text[3:])
		if (i == self.ID_GREEN):
			g = self.green.GetValue()
			sr = ''
			if (g < 16):
				sr = '0'
			sr = sr + hex(g)[2:]
			sr = sr.upper()
			text = self.txtColor.GetValue()
			if (len(text) is not 7):
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("The text box above should be formatted as follows:\n#0033AA,\nStart with the # character, followed by 6 characters which can be 0-9 or a-f.\nDrPython will ignore any changes to the color until this is fixed."), "Error")
				d.ShowModal()
				d.Destroy()
				return
			self.txtColor.SetValue(text[0:3] + sr + text[5:])
		if (i == self.ID_BLUE):
			b = self.blue.GetValue()
			sr = ''
			if (b < 16):
				sr = '0'
			sr = sr + hex(b)[2:]
			sr = sr.upper()
			text = self.txtColor.GetValue()
			if (len(text) is not 7):
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("The text box above should be formatted as follows:\n#0033AA,\nStart with the # character, followed by 6 characters which can be 0-9 or a-f.\nDrPython will ignore any changes to the color until this is fixed."), "Error")
				d.ShowModal()
				d.Destroy()
				return
			self.txtColor.SetValue(text[0:5] + sr + text[7:])

	def OnTextChange(self, event):
		v = self.txtColor.GetValue()
		if (len(v) < 1):
			self.red.Enable(False)
			self.green.Enable(False)
			self.blue.Enable(False)
			return
		if (v[0] == "#") and (len(v) == 7):
			#Update Sliders
			self.red.Enable(True)
			self.green.Enable(True)
			self.blue.Enable(True)
			try:
				self.red.SetValue(int(v[1:3], 16))
				self.green.SetValue(int(v[3:5], 16))
				self.blue.SetValue(int(v[5:7], 16))
			except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Bad Color Data.  Should be in the form #00A3F4\nUse only digits 0-9, characters A-F"), "Error")
				d.ShowModal()
				d.Destroy()					
			#Update Parent
			if (self.isForeground):
				self.GetParent().foreground = v
			else:
				self.GetParent().background = v
			self.GetParent().SetColor()
		else:
			self.red.Enable(False)
			self.green.Enable(False)
			self.blue.Enable(False)

class drStyleDialog(wx.Dialog):

	def __init__(self, parent, id, title, stylestring, defaultstylestring, ChangeSpec = 0):

		wx.Dialog.__init__(self, parent, id, title, wx.Point(50, 50), wx.Size(425, 450))

		self.ID_CANCEL = 101
		self.ID_OK = 102
		
		self.ID_FONTS = 103
		self.ID_SIZE = 104
			
		self.ID_BOLD = 109
		self.ID_ITALIC = 110
		self.ID_UNDERLINE = 111
		
		if (wx.Platform == '__WXMSW__'):
			BTN_Y_POS = 380
		else:
			BTN_Y_POS = 400

		self.Enum = wx.FontEnumerator()
		self.Enum.EnumerateFacenames()
		self.FontList = self.Enum.GetFacenames()
		self.FontList.sort()

		self.OK = False

		self.font = getStyleProperty("face", stylestring)
		if (len(self.font) <= 0):
			self.font = getStyleProperty("face", defaultstylestring)
		self.size = getStyleProperty("size", stylestring)
		if (len(self.size) <= 0):
			self.size = getStyleProperty("size", defaultstylestring)
		self.foreground = getStyleProperty("fore", stylestring)
		if (len(self.foreground) <= 0):
			self.foreground = getStyleProperty("fore", defaultstylestring)
		self.background = getStyleProperty("back", stylestring)
		if (len(self.background) <= 0):
			self.background = getStyleProperty("back", defaultstylestring)
		self.bold = getStyleProperty("bold", stylestring)
		self.italic = getStyleProperty("italic", stylestring)
		self.underline = getStyleProperty("underline", stylestring)
		self.txtPreview = wx.stc.StyledTextCtrl(self, id, wx.Point(225, 15), wx.Size(175, 150))

		if (ChangeSpec > 0):
			self.font = getStyleProperty("face", defaultstylestring)
			self.size = getStyleProperty("size", defaultstylestring)
			if (ChangeSpec == 1):
				self.background = getStyleProperty("back", defaultstylestring)
			elif (ChangeSpec == 3):
				self.foreground = getStyleProperty("fore", defaultstylestring)

		try:
			self.FontList.index(self.font)
		except:
				d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Current Font is not in the fontlist.  \nSetting to the default."), "Error")
				d.ShowModal()
				d.Destroy()						
				self.font = getStyleProperty("face", drPreferences(self.GetParent().GetGrandParent().prefs.platform_is_windows).txtDocumentStyleArray[0])
				self.size = getStyleProperty("size", drPreferences(self.GetParent().GetGrandParent().prefs.platform_is_windows).txtDocumentStyleArray[0])

		self.txtPreview.SetText("Preview\n()[]{}\n0123")
		self.txtPreview.SetUseHorizontalScrollBar(0)

		self.txtPreview.SetReadOnly(1)
		self.txtPreview.SetMarginWidth(0, 0)
		self.txtPreview.SetMarginWidth(1, 0)
		self.txtPreview.SetMarginWidth(2, 0)

		self.txtPreview.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, ("fore:" + self.foreground + ",back:" + self.background + ",size:" + self.size + ",face:" + self.font + "," + self.bold + "," + self.italic + "," + self.underline))

		self.txtPreview.StyleClearAll()

		self.txtPreview.StartStyling(0, 0xff)
							
		self.boxFonts = wx.ListBox(self, self.ID_FONTS, wx.Point(10, 10), wx.Size(150, 150), self.FontList)
		
		try:
			self.boxFonts.SetStringSelection(self.font)
		except:
			d = drScrolledMessageDialog.ScrolledMessageDialog(self, ("Something awful happened trying to \nset the font to the default."), "Error")
			d.ShowModal()
			d.Destroy()	
			self.boxFonts.SetSelection(0)

		self.boxSize = wx.ComboBox(self, self.ID_SIZE, self.size, wx.Point(10, 175), wx.Size(150, 50), ["8", "10", "12", "14", "16", "18", "20", "22", "24", "26", "28", "30"])
	
		self.fgPanel = drColorPanel(self, id, wx.Point(10, 200), wx.Size(200, 140), self.foreground, True)
		self.bgPanel = drColorPanel(self, id, wx.Point(220, 200), wx.Size(200, 140), self.background, False)

		self.chkBold = wx.CheckBox(self, self.ID_BOLD, "Bold", wx.Point(10, 345))
		if (len(self.bold) > 0):
			self.chkBold.SetValue(1)
		self.chkItalic = wx.CheckBox(self, self.ID_ITALIC, "Italic", wx.Point(110, 345))
		if (len(self.italic) > 0):
			self.chkItalic.SetValue(1)
		self.chkUnderline = wx.CheckBox(self, self.ID_UNDERLINE, "Underline", wx.Point(210, 345))
		if (len(self.underline) > 0):
			self.chkUnderline.SetValue(1)

		if (ChangeSpec > 0):
			self.boxFonts.Enable(False)
			self.boxSize.Enable(False)
			self.chkBold.Enable(False)
			self.chkItalic.Enable(False)
			self.chkUnderline.Enable(False)
			if (ChangeSpec == 1):
				self.bgPanel.Enable(False)
			elif (ChangeSpec == 3):
				self.fgPanel.Enable(False)
				
		self.btnCancel = wx.Button(self, self.ID_CANCEL, "&Cancel", wx.Point(5, BTN_Y_POS), wx.Size(-1, -1))
		self.btnCancel.SetDefault()	
		self.btnOk = wx.Button(self, self.ID_OK, "&Ok", wx.Point(300, BTN_Y_POS), wx.Size(-1, -1))

		self.Bind(wx.EVT_LISTBOX,  self.OnFontSelect, id=self.ID_FONTS)
		self.Bind(wx.EVT_COMBOBOX,  self.OnSizeSelect, id=self.ID_SIZE)
		self.Bind(wx.EVT_CHECKBOX,  self.OnBold, id=self.ID_BOLD) 
		self.Bind(wx.EVT_CHECKBOX,  self.OnItalic, id=self.ID_ITALIC) 
		self.Bind(wx.EVT_CHECKBOX,  self.OnUnderline, id=self.ID_UNDERLINE)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnCancel, id=self.ID_CANCEL)
		self.Bind(wx.EVT_BUTTON,  self.OnbtnOk, id=self.ID_OK)			

	def OnFontSelect(self, event):
		self.font = self.boxFonts.GetStringSelection()
		self.txtPreview.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, ("fore:" + self.foreground + ",back:" + self.background + ",size:" + self.size + ",face:" + self.font + "," + self.bold + "," + self.italic + "," + self.underline))
		self.txtPreview.StyleClearAll()
		self.txtPreview.StartStyling(0, 0xff)

	def OnSizeSelect(self, event):		
		self.size = self.boxSize.GetStringSelection()
		self.txtPreview.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, ("fore:" + self.foreground + ",back:" + self.background + ",size:" + self.size + ",face:" + self.font + "," + self.bold + "," + self.italic + "," + self.underline))
		self.txtPreview.StyleClearAll()
		self.txtPreview.StartStyling(0, 0xff)

	def SetColor(self):
		self.txtPreview.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, ("fore:" + self.foreground + ",back:" + self.background + ",size:" + self.size + ",face:" + self.font + "," + self.bold + "," + self.italic + "," + self.underline))
		self.txtPreview.StyleClearAll()
		self.txtPreview.StartStyling(0, 0xff)

	def OnBold(self, event):
		if (self.chkBold.IsChecked()):
			self.bold = "bold"
		else:
			self.bold = ""
		self.txtPreview.StyleResetDefault()
		self.txtPreview.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, ("fore:" + self.foreground + ",back:" + self.background + ",size:" + self.size + ",face:" + self.font + "," + self.bold + "," + self.italic + "," + self.underline))
		self.txtPreview.StyleClearAll()
		self.txtPreview.StartStyling(0, 0xff)

	def OnItalic(self, event):
		if (self.chkItalic.IsChecked()):
			self.italic = "italic"
		else:
			self.italic = ""
		self.txtPreview.StyleResetDefault()
		self.txtPreview.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, ("fore:" + self.foreground + ",back:" + self.background + ",size:" + self.size + ",face:" + self.font + "," + self.bold + "," + self.italic + "," + self.underline))
		self.txtPreview.StyleClearAll()
		self.txtPreview.StartStyling(0, 0xff)

	def OnUnderline(self, event):
		if (self.chkUnderline.IsChecked()):
			self.underline = "underline"
		else:
			self.underline = ""
		self.txtPreview.StyleResetDefault()
		self.txtPreview.StyleSetSpec(wx.stc.STC_STYLE_DEFAULT, ("fore:" + self.foreground + ",back:" + self.background + ",size:" + self.size + ",face:" + self.font + "," + self.bold + "," + self.italic + "," + self.underline))
		self.txtPreview.StyleClearAll()
		self.txtPreview.StartStyling(0, 0xff)

	def OnbtnCancel(self, event):
		self.EndModal(0)

	def OnbtnOk(self, event):
		self.OK = True
		self.EndModal(0)

	def ClickedOk(self):
		return self.OK

	def GetBackground(self):
		return self.background

	def GetColorString(self):
		return ("fore:" + self.foreground + ",back:" + self.background)
		
	def GetForeground(self):
		return self.foreground

	def GetStyleString(self):
		return ("fore:" + self.foreground + ",back:" + self.background + ",size:" + self.size + ",face:" + self.font + "," + self.bold + "," + self.italic + "," + self.underline)